/******************************************************************************
 * $Id: include/lcd.h, v0.01 17:45 27.11.2010 TiN Exp $
 *
 * Copyright (c) 2010, xDevs.com ARM Microcontroller Software Support
 *
 * LCD ST7588-based specific definitions
 *
 * Project maintainers:
 *  o Tsemenko Ilya  (ilya@xdevs.com)
 *  o Alex Gurin     (warder@xdevs.com)
 */

#ifndef LCD_H
#define LCD_H 

#include <types.h>

#define LCD_ID              1

#define CELL_TAB_LENGTH     88   /* Cells number 6x8 pixel */

/**
  * LCD Function set commands 
  */
#define LCFS_H00            0x10 /* Set extended table 00 */
#define LCFS_H01            0x11 /* Set extended table 01 */
#define LCFS_H10            0x12 /* Set extended table 10 */
#define LCFS_H11            0x13 /* Set extended table 11 */
#define LCFS_POWER_DOWN     0x14 /* Chip is power down mode */
#define LCFS_MIRROR_Y       0x18 /* reverse dir SEG (131->0) */
#define LCFS_MIRROR_X       0x30 /* reverse dir COM (79->0) */
#define LCFS_MIRROR_XY      0x38 /* reverse both dir */


/**
  * EXT table 00 commands 
  */
#define LXT0_PRS_LOW        0x04 
#define LXT0_PRS_HIGH       0x05
#define LXT0_RMW_END        0x06
#define LXT0_RMW            0x07
#define LXT0_DISPLAY_OFF    0x08 /* Display pixels disabled */
#define LXT0_DISPLAY_FILLED 0x09 /* Display pixels totally on */
#define LXT0_DISPLAY_NORMAL 0x0C /* Display pixels normal used */
#define LXT0_DISPLAY_INV    0x0D /* Display pixels inverted */

#define LXT0_SET_ADDR_Y(y)  ((y && 0x0F) | (1 << 6))
#define LXT0_SET_ADDR_XL(x) ((x && 0x0F) | 0xE0)
#define LXT0_SET_ADDR_XH(x) (((x && 0xF0) >> 4) | 0xF0)


/**
  * EXT table 01 commands
  */
#define LXT1_CONF_MBT_HADDR 0x08
#define LXT1_CONF_MBT_VADDR 0x09
#define LXT1_CONF_LBT_HADDR 0x0C
#define LXT1_CONF_LBT_VADDR 0x0D
#define LXT1_BIAS_1_100     0x10
#define LXT1_BIAS_1_81      0x11
#define LXT1_BIAS_1_65_68   0x12
#define LXT1_BIAS_1_49      0x13
#define LXT1_BIAS_1_40_36   0x14
#define LXT1_BIAS_1_24      0x15
#define LXT1_BIAS_1_18_16   0x16
#define LXT1_BIAS_1_10_9_8  0x17

#define LXT1_VOP(v)         ((v & 0x7F) | (1 << 7))


/**
  * EXT table 02 commands
  */
#define LXT2_PARTAL_SCREEN  0x05
#define LXT2_PARTAL_32R_SZ  0x09

#define LXT2_PARTAL_PART(p) ((p & 0x0F) | (1 << 4))
#define LXT2_PARTAL_STLN(l) ((l & 0x7F) | (1 << 7))


/**
  * EXT table 03 valid vals
  */
#define FPS50               0x00
#define FPS68               0x01
#define FPS70               0x02
#define FPS73               0x03
#define FPS75               0x04
#define FPS78               0x05
#define FPS80               0x06
#define FPS150              0x07


/**
  * EXT table 03 commands
  */
#define LXT3_SWRST          0x03
#define LXT3_HIGH_POWER     0xB4

#define LXT3_FRAMERATE(f)   ((f && 0x07) | (1 << 3))
#define LXT3_INVERT(ln)     ((ln && 0x1F) | (1 << 6))


typedef struct hc1_lcd_display_s    hc1_lcd_display_t;
typedef struct hc1_lcd_cell_value_s hc1_lcd_cell_value_t;
typedef struct hc1_lcd_icon_value_s hc1_lcd_icon_value_t;

struct hc1_lcd_cell_value_s {
    uint8_t bitvl[6];             /* 8x6 pixels lcd area */
};

struct hc1_lcd_display_s {
    hc1_lcd_cell_value_t    cell_vtab[CELL_TAB_LENGTH];
};

extern int lcd_init(void);
extern void hc1_lcd_refresh_display(void);
extern int  hc1_lcd_update_cell_value(uint8_t ci, hc1_lcd_cell_value_t *cv);
int hc1_graph(void);

#endif /* LCD_H */
